var geomap = (function () {
	var base_url;
	var edit_mode;
	var map;
	var markers = [];
	var lines = [];
	var geocoder;
	var popups = [];
	var new_marker;
	var states = {
		'host':{
			0:'up',
			1:'down',
			2:'unreachable',
			6:'pending'},
		'service':{
			0:'ok',
			1:'warning',
			2:'critical',
			3:'unknown',
			6:'pending'}};
	var icons = {};
	var colors = {
		'host':['#00FF00', '#FF0000', '#FF9900', '#FF9900', , , '#999999'],
		'service':['#00FF00', '#FFFF00', '#FF0000', '#FF9900', , , '#999999']};
	var search_markers = [];

	return {
		type: "google",
		init: function(options){
			base_url = options.base_url;
			edit_mode = options.edit_mode;

			map = new google.maps.Map(document.getElementById('map'), {
				zoom:options.zoom,
				minZoom:2,
				center:new google.maps.LatLng(options.center[0], options.center[1]),
				mapTypeId:google.maps.MapTypeId.ROADMAP,
				disableDoubleClickZoom:true
			});

			if (edit_mode) {
				geocoder = new google.maps.Geocoder();
				google.maps.event.addListener(map, 'dblclick', function (event) {
					if (new_marker)
						new_marker.setMap(null);
					new_marker = new google.maps.Marker({map:map, position:event.latLng});
					var infowindow = new google.maps.InfoWindow({content:'<a href="' + base_url + 'index.php/geomap/index?edit=1" '
							+ 'class="add_location" latlng="' + event.latlng.lat + ',' + event.latlng.lng + '">Add location</a>'
							+ '<br><a href="#" onclick="geomap.onCancel(); return false;">Cancel</a>'
							+ '<div style="clear: both; float: none; height: 1px;"></div>'});
					google.maps.event.addListener(infowindow, 'closeclick', function () {
						new_marker.setMap(null);
						new_marker = undefined;
					});
					infowindow.open(map, new_marker);
				});
			}

			for (var type in states) {
				icons[type] = {};
				for (var state in states[type]) {
					icons[type][state] = new google.maps.MarkerImage(
						base_url + 'application/views/icons/16x16/shield-' + states[type][state] + '.png',
						new google.maps.Size(16, 16),
						new google.maps.Point(0, 0),
						new google.maps.Point(8, 8)
					);
				}
			}
		},
		populate:function (locations, links) {
			var bindPopup = function (anchor, text) {
				var infowindow = new google.maps.InfoWindow({content:text});
				google.maps.event.addListener(anchor, 'click', function (event) {
					if (anchor instanceof google.maps.Marker)
						infowindow.open(map, anchor);
					else {
						infowindow.setPosition(event.latLng);
						infowindow.open(map);
					}
				});
				popups.push(infowindow);
			};

			for (var i in locations) {
				var location = locations[i];
				var coords = location.point.split(',');

				var info = '<strong>' + location.label + '</strong>';
				if (location.description)
					info += '<br>' + location.description;
				var base_type = '';
				if (location.object && location.object.type) {
					base_type = location.object.type;
					if (!states[base_type])
						base_type = base_type.slice(0, -5);
				}
				if (location.object) {
					var object = location.object;
					info += '<br>' + object.type + ': <a href="' + geomap.onDetails(object) + '"><strong>'
						+ (object.type == 'service' ? object.description + '</strong> at <strong>' + object.host : object.name)
						+ '</strong></a><br>state: <strong>' + states[base_type][object.state] + '</strong>';
				}
				if (edit_mode) {
					info += '<br><a href="#" onclick="geomap.onLink(\'' + location.id + '\'); return false;">Link</a>'
						+ '<br><a class="edit_location" href="' + base_url + 'index.php/geomap/index?edit=1" location_id="' + location.id + '">Edit</a>'
						+ '<br><a href="#" class="remove_location" location_id="' + location.id + '">Remove</a>'
						+ '<div style="clear: both; float: none; height: 1px;"></div>';
				}
				var options = {
					map:map,
					position:new google.maps.LatLng(parseFloat(coords[0]), parseFloat(coords[1]))
				};
				if (location.object && (location.object.state !== undefined) && icons[base_type][location.object.state]) {
					options.icon = icons[base_type][location.object.state];
					options.zIndex = google.maps.Marker.MAX_ZINDEX+location.object.state;
				} else {
					options.icon = icons['host'][6];
				}
				var marker = new google.maps.Marker(options);
				bindPopup(marker, info);
				markers.push(marker);
			}

			for (var i in links) {
				var link = links[i];
				var coords1;
				var coords2;

				var base_type = '';
								if (link.object && link.object.type) {
									base_type = link.object.type;
									if (!states[base_type])
										base_type = base_type.slice(0, -5);
								}


				if (locations[link.id1] && locations[link.id2]) {
					coords1 = locations[link.id1].point.split(',');
					coords2 = locations[link.id2].point.split(',');
				}
				else
					continue;

				var info = '<strong>' + link.description + '</strong>';
				if (link.object) {
					var object = link.object;
					info += '<br>' + object.type + ': <a href="' + geomap.onDetails(object) + '"><strong>'
						+ (object.type == 'service' ? object.description + '</strong> at <strong>' + object.host : object.name)
						+ '</strong></a><br>state: <strong>' + states[base_type][object.state] + '</strong>';
				}
				if (edit_mode) {
					info += '<br><a href="#" onclick="geomap.onEditLink(\'' + link.id + '\'); return false;">Edit</a>'
						+ '<br><a href="#" onclick="geomap.onRemoveLink(\'' + link.id + '\'); return false;">Remove</a>';
				}

				var line = new google.maps.Polyline({map:map, path:[
					new google.maps.LatLng(parseFloat(coords1[0]), parseFloat(coords1[1])),
					new google.maps.LatLng(parseFloat(coords2[0]), parseFloat(coords2[1]))
				],
					strokeColor: link.object && link.object.state != undefined && colors[base_type][link.object.state] ? colors[base_type][link.object.state] : colors['host'][6],
					strokeOpacity:0.5,
					strokeWeight:5
				});
				bindPopup(line, info);
				lines.push(line);
			}
		},
		clear:function () {
			this.closeNew();

			for (var i = 0; i < search_markers.length; i++) {
				search_markers[i].setMap(null);
				delete search_markers[i];
			}
			search_markers = [];

			for (var i = 0; i < lines.length; i++) {
				lines[i].setMap(null);
				delete lines[i];
			}
			lines = [];

			for (var i = 0; i < markers.length; i++) {
				markers[i].setMap(null);
				delete markers[i];
			}
			markers = [];
		},
		closeNew:function () {
			for (var i = 0; i < popups.length; i++) {
				popups[i].close();
				delete popups[i];
			}
			popups = [];

			if (new_marker) {
				new_marker.setMap(null);
				new_marker = undefined;
			}
		},
		search:function (address, callback) {
			geocoder.geocode({'address':address}, function (result, status) {
				var ret_val = [];
				if (status === google.maps.GeocoderStatus.OK) {
					$.each(result, function (val) {
						ret_val.push([this.geometry.location.lat(), this.geometry.location.lng()]);
					});
				}
				else
					ret_val = false;
				callback.call(this, ret_val);
			});
		},
		showSearchMarkers:function (list) {
			if (!edit_mode)
				return false;

			var mark;
			while (mark = search_markers.shift())
				mark.setMap(null);

			for (var i = 0; i < list.length; i++) {
				var marker = new google.maps.Marker({
					'position':new google.maps.LatLng(list[i][0], list[i][1]),
					'map':map
				});
				(function (indx, marker) {
					google.maps.event.addListener(marker, 'click', function (_event_) {
						if (typeof(new_marker) !== 'undefined') {
							new_marker.setMap(null);
							delete new_marker;
						}
						if (typeof(marker.__openedInfoWindow) !== 'undefined')
							marker.__openedInfoWindow.close();

						var infowindow = new google.maps.InfoWindow({content:'<a href="' + base_url + 'index.php/geomap/index?edit=1" '
							+ 'class="add_location" latlng="' + list[indx][0] + ',' + list[indx][1] + '">Add location</a>'
							+ '<br><a href="#" onclick="geomap.onCancel(); return false;">Cancel</a>'
							+ '<div style="clear: both; float: none; height: 1px;"></div>'});
						infowindow.open(map, marker);
						marker.__openedInfoWindow = infowindow;
						new_marker = marker;
					});
				})(i, marker);
				search_markers.push(marker);
			}
		},
		getViewPoint:function () {
			if(!map || !map.getCenter()) {
				// we don't have a map, needed to check because
				// cucumber does not tolerate failure
				return null;
			}
			return {
				center:[map.getCenter().lat(), map.getCenter().lng()],
				zoom:map.getZoom()
			};
		}
	};
})();
